@@ -5,7 +5,7 @@
 # Author    : Chris Jones
 # Email     : crj341@student.bham.ac.uk
 # Date      : 26/06/2020
-# Version   : 0.3.27
+# Version   : 0.3.31
 
 from operator import contains
 import  numpy                   as      np
@@ -149,6 +149,8 @@ class AFM:
         Funciton to calculate the adhesion value for each curve by finding the minimum
         point on the retact curve. A histogram of calculated values can be plotted by
         setting the 'plot_hist' input to 'True'
+    calc_work_adhesion:
+        Function to calculate the work of adhesion (J)
     overlay_adhesion_hist:
         Funciton to plot a histogram comparing adhesion values for multiple experiments.
         A separate instance of the AFM class containing the data for each should be
@@ -211,8 +213,8 @@ class AFM:
         self.z_approach_piezo =[]
         self.z_withdraw_piezo =[]
 
-
         self.adhesion = []
+        self.work_adhesion = []
         self.modulus = []
         self.contact_point = []
         self.indent_depth = []
@@ -956,6 +958,7 @@ class AFM:
             indent_depth = self.indent_depth,
             snap_in = self.snap_in,
             snap_dist = self.snap_dist,
+            work_adhesion = self.work_adhesion,
             max_force = self.max_force
         )
 
@@ -999,6 +1002,7 @@ class AFM:
         self.snap_in = data['snap_in']
         self.snap_dist = data['snap_dist']
         self.max_force = data['max_force']
+        self.work_adhesion = data['work_adhesion']
 
     def calc_adhesion(
         self,
@@ -1041,6 +1045,31 @@ class AFM:
         print('Mean adhesion = %(mean).2f nN' % {'mean': np.mean(self.adhesion)})
         print('Standard deviation = %(std).2f nN' % {'std': np.std(self.adhesion)})
 
+    def calc_work_adhesion(
+        self
+    ):
+        '''
+        Function to calculate work of adhesion (J).
+        '''
+        work = np.zeros(self.z_approach.shape[1])
+
+        for curve_number in range(0, self.z_approach.shape[1]):
+            force = self.retract[:,curve_number]
+            distance = self.z_withdraw[:,curve_number]
+
+            distance = distance[~np.isnan(force)]
+            force = force[~np.isnan(force)]
+
+            d = np.linspace(distance[0], distance[-1], 10000)
+            f = np.interp(d, distance, force)
+
+            d = d[f < 0]
+            f = f[f < 0]
+            
+            work[curve_number] = np.trapz(f, d) * -1
+
+        self.work_adhesion = work
+
     def overlay_adhesion_hist(
         self,
         *all_adhesion,
@@ -1528,14 +1557,17 @@ class AFM:
         return_fig = False,
         auto_cell = True,
         plot = 'adhesion',
+        cmin = 0,
+        cmax = 1,
+        set_c_scale = False,
         topography_3d = False
     ):
 
-        plot_types = ['adhesion', 'modulus', 'topography', 'indent', 'max_force', 'snap_in','snap_dist']
+        plot_types = ['adhesion', 'modulus', 'topography', 'indent', 'max_force', 'snap_in','snap_dist','work']
         member = plot in plot_types
 
         if member == False:
-            print('Incorrect plot type specified. Available plots are adhesion, modulus, topography, indent, max_force, snap_in and snap_dist.')
+            print('Incorrect plot type specified. Available plots are adhesion, modulus, topography, indent, max_force, work, snap_in and snap_dist.')
 
         else:
 
@@ -1556,6 +1588,7 @@ class AFM:
             force = np.zeros(x_cells * y_cells)
             snap = np.zeros(x_cells * y_cells)
             snap_dist  = np.zeros(x_cells * y_cells)
+            work  = np.zeros(x_cells * y_cells)
 
             x_counter = 1
             y_counter = 1
@@ -1586,6 +1619,8 @@ class AFM:
 
                 if plot == 'snap_dist':
                     snap_dist[j] = self.snap_dist[j]
+                if plot == 'work':
+                    work[j] = self.work_adhesion[j]
 
                 height[j] = self.contact_point[j]
 
@@ -1620,6 +1655,9 @@ class AFM:
                     if plot == 'snap_dist':
                         snap_dist[j] = self.snap_dist[j]
 
+                    if plot == 'work':
+                        work[j] = self.work_adhesion[j]
+
                     height[j] = self.contact_point[j]
 
                     if updown == 1:
@@ -1665,17 +1703,32 @@ class AFM:
             elif plot == 'snap_dist':
                 plt_title = 'Snap in distance (nm)'
                 plt_data = snap_dist
+            elif plot == 'work':
+                plt_title = 'Work of adhesion (J)'
+                plt_data = snap_dist
 
             if topography_3d == False:
 
-                fig = go.Figure(data=go.Heatmap(
-                        x = x_pos,
-                        y = y_pos,
-                        z = plt_data,
-                        type = 'heatmap',
-                        colorscale = 'YlOrBr',
-                        colorbar=dict(title=plt_title)
-                        ))
+                if set_c_scale is False:
+                    fig = go.Figure(data=go.Heatmap(
+                            x = x_pos,
+                            y = y_pos,
+                            z = plt_data,
+                            type = 'heatmap',
+                            colorscale = 'YlOrBr',
+                            colorbar=dict(title=plt_title)
+                            ))
+                elif set_c_scale is True:
+                    fig = go.Figure(data=go.Heatmap(
+                            x = x_pos,
+                            y = y_pos,
+                            z = plt_data,
+                            cmin = cmin,
+                            cmax = cmax,
+                            type = 'heatmap',
+                            colorscale = 'YlOrBr',
+                            colorbar=dict(title=plt_title)
+                            ))
 
                 fig.update_layout(yaxis = dict(scaleanchor = 'x'))
                 fig.update_layout(plot_bgcolor='rgba(0,0,0,0)')
@@ -1721,8 +1774,13 @@ class AFM:
                     z3[int(x_grid[i])-1 , int(y_grid[i])-1] = height[i]
                     d3[int(x_grid[i])-1 , int(y_grid[i])-1] = plt_data[i]
 
-                fig = go.Figure(go.Surface(x=x3, y=y3, z=z3, surfacecolor=d3,
-                                colorbar=dict(title=plt_title)))
+                if set_c_scale is False:
+                    fig = go.Figure(go.Surface(x=x3, y=y3, z=z3, surfacecolor=d3,
+                                    colorbar=dict(title=plt_title)))
+                elif set_c_scale is True:
+                    fig = go.Figure(go.Surface(x=x3, y=y3, z=z3, surfacecolor=d3,
+                                    cmin = cmin, cmax = cmax,
+                                    colorbar=dict(title=plt_title)))
 
                 fig.update_layout(scene = dict(
                                 xaxis_title='X Position (μm)',
